#!/bin/bash

################################################################################
# AUTOR: 
# CRIADO EM: 
#
# REVISÕES:
# --------------------------------------------------------------
# <versão> <Data da Revisão>, <autor da revisao>
#     - <Descrição das modificações>
#
################################################################################

readonly VERSION="1.0"
readonly OK=0
readonly WARNING=1
readonly CRITICAL=2
readonly UNKNOWN=3
readonly TRUE=1
readonly FALSE=0
readonly START_INTERVAL_DEFAULT=0
readonly END_INTERVAL_DEFAULT=100

# declare -r CONST.

programName=$( basename $0 )
perfdata=${FALSE}
warning=""
critical=""
verbose=${FALSE}


################################################################################
# Exibi o manual do software
################################################################################

function getGuideLine {

    cat <<EOF

------------------------------------------
Manual de exemplo.
------------------------------------------

Descrição do plugin:
    Este plugin monitora o valor gerado randomicamente pelo sistema.

Instalação:
    Passos para instalação

Dependências:
    <Listar todas>
    <Dependência>
    <Instalação>

Funcionamento:
    O plugin recebe do usuário um intervalo no qual deve gerar um numero de forma
    randômica e caso o valor seja maior que o threshold ele alarma.

Exemplos de execução e retorno do comando:
    <Listar todos possíveis>

    ./check_example -s 1 -e 15
    OK: Numero gerado 10

    ./check_example -s 1 -e 15 --perfdata
    OK: Numero gerado 15 | num=15;;;;

    ./check_example -s 1 -e 15 -w 10 -c 11
    CRITICAL: Numero gerado 15 | num=15;10;11;;

    ./check_example -s 1 -e 15 -w 10 -c 20
    WARNING: Numero gerado 15 | num=15;10;20;;

    ./check_example -s 20 -e 15
    UNKNOWN: Intervalo invalido


EOF

}

################################################################################
# Mostrar o help
################################################################################

function usage(){

    echo -e "
Usage:
	${programName} -s <start_number> -e <end_number>
\t\t[-w <warning>] [-c <critical>]
"

}

################################################################################
# Mostra a versão do software
################################################################################

function version(){
    echo ${programName} ${VERSION} ${URL}
}

function help(){
    local usage=$( usage )
    local version=$( version )
    cat <<EOF

${version}

Developed by:
    OpServices

Author:
    Daniel Ortiz - daniel.ortiz@opservices.com

${usage}

 --usage
   Print usage information
 -h, --help
   Print detailed help screen
 -V, --version
   Print version information
 -s, --start=INTEGER
   Initial interval number
 -e, --end=INTEGER
   End interval number
 -w, --warning=INTEGER
   Warning value
 -c, --critical=INTEGER
   Critical value
 --perfdata
   Show perfdata
 --man
   Show guideline
 -t, --timeout=INTEGER
   Seconds before plugin times out (default: 15)
 -v, --verbose
   Show details for command-line debugging (can repeat up to 3 times)


EOF

}



################################################################################
# Seta variaveis definidas conforme parâmetros passados
################################################################################

function getOptions(){

    ARG="$@"

    eval set -- "$@"

    while true ; do
        case "$1" in
            --usage)
                usage
                exit ${OK}
                ;;
            -h|--help)
                help
                exit ${OK}
                ;;
            -s|--start)
                start=${2}
                shift 2
                ;;
            -e|--end)
                end=${2}
                shift 2
                ;;
            -w|--warning)
                warning=${2}
                shift 2
                ;;
            -c|--critical)
                critical=${2}
                shift 2
                ;;
            --perfdata)
                perfdata=${TRUE}
                shift 2
                ;;
            --man)
                getGuideLine
                exit ${OK}
                ;;
            -v|--verbose)
                verbose=${TRUE}
                shift 2
                ;;
            *)
                shift
                break;;
        esac
    done

    start=${start:=$START_INTERVAL_DEFAULT}
    end=${end:=$END_INTERVAL_DEFAULT}

}

################################################################################
# Gera um numero randômico entre dois valores passado por parâmetro.
# O numero gerado segue o seguinte padrão: x >= start e x <= end
# @link http://stackoverflow.com/questions/1952404/linux-bash-multiple-variable-assignment
# @link http://www.cyberciti.biz/faq/bash-shell-script-generating-random-numbers/
# @param $start Valor inicial do intervalo
# @param $end Valor final do intervalo
# @return $randomNumber Numero randômico gerado.
################################################################################


function getRandomNumber {

    local start
    local end

    read start end <<< $@

    test ${start} -gt ${end} && echo "Intervalo inválido" && exit ${UNKNOWN}

    local randomNumber=$(( ((${RANDOM} % (${end} - ${start})) + 1) + ${start} ))

    if [ ${verbose} -eq ${TRUE} ]; then
		echo "Debug: Start interval ${start}" 1>&2
		echo "Debug: End interval ${end}" 1>&2
	    echo "Debug: Numero Gerado ${randomNumber}" 1>&2
	fi

    echo ${randomNumber}
}

################################################################################
# # Verifica qual status da execução do plugin segundo os thresholds e value
# passado
# @param $value Numero que deve ser utilizado para fazer os testes e determinar
#               o status
# @return $status Codigo de saída para o plugin
################################################################################


function checkThreshold(){
    local value=${1}
    local status=${OK}

    warning=${warning:=$END_INTERVAL_DEFAULT}
    critical=${critical:=$END_INTERVAL_DEFAULT}

    if [ ${value} -ge ${warning} -a ${value} -lt ${critical} ]; then
        status=${WARNING}
    elif [ ${value} -ge ${critical} ]; then
        status=${CRITICAL}
    fi

    echo ${status}
}

################################################################################
# Rotina principal do programa
################################################################################

function main() {

    getOptions $(
        getopt -a -o ?hVs:e:w:c:??v -l "usage help version start: end:\
        warning: critical: perfdata man verbose" -n "$0" -- "$@"
    )


    local randomNumber=$( getRandomNumber ${start} ${end} )
    local outputMessage="Numero gerado ${randomNumber}"
    local outputPerfData=""

    if [ "${perfdata}" -eq ${TRUE} ] || [ -n "${warning}" -a -n "${critical}" ]
    then
        outputPerfData="| num=${randomNumber};${warning};${critical};0;"
    fi

    echo ${outputMessage} ${outputPerfData}
    exit $(checkThreshold ${randomNumber})
}

main "$@"

# vim: set smartindent tabstop=4 shiftwidth=4 softtabstop=4 expandtab[ENTER] #
